%_______________________________________________________________________
%
% Run random dot motion experiment defined in createDotInfo
%_______________________________________________________________________
%
% Input
%
% dotInfo | experiment configuration (struct)
%    subj | identifier of subject (string)
% condSet | conditions (cell array of strings)
%_______________________________________________________________________
%
% 170707    | updated palamedes to take in more inputs
% 170809    | adapted to state switch task practice (not parameter estimation part)

function [thresholds] = MAT_practice_170809(dotInfo, subj, setup, P)

  %%
  %% sanitize function parameters
  %%
  if nargin == 0
    eval('help dotsExperiment')
    return
  end

  if not(exist('dotInfo','var'))
    error('Missing input: dotInfo')
  elseif isempty(dotInfo) || not(isstruct(dotInfo))
    error('Invalid input: dotInfo (requires non-empty struct)')
  end

  if not(exist('subj','var'))
    error('Missing input: subj')
  elseif not(isempty(subj)) && ischar(subj)
    dotInfo.subj = subj;
  else
    error('Invalid input: subj (requires non-empty string)')
  end

  if not(isfield(dotInfo,'deployed'))
    dotInfo.deployed = true;
  end

  %%
  %% for saving behavioural data
  %%

  sessionFile = [dotInfo.dataDir, subj '_' datestr(now, 'yymmdd_HHMM'), '.mat'];
    
  %%
  %% prepare presentation
  %%
  displayList = Screen('Screens'); % There appears to be a problem in DEBUG mode.
  displayIdx = max(displayList);
  KbName('UnifyKeyNames'); % use portable key naming scheme
  oldVerbosityLevel = Screen('Preference', 'Verbosity', 2); % show errors and warning

    if setup.DEBUG == 1
        Screen('Preference', 'SkipSyncTests', 1);
        PsychDebugWindowConfiguration(0, setup.opacity);
    else
        clear Screen; %PsychDebugWindowConfiguration([], 1);
    end

  try

    %%
    %% prepare canvas
    %%
    screenInfo = openExperiment(50,50,displayIdx); clc; % open drawing canvas
    if numel(Screen('screens')) == 1 || dotInfo.deployed
      HideCursor(screenInfo.curWindow);
    end
    Screen('TextFont', screenInfo.curWindow, 'Helvetica');
    Screen('TextSize', screenInfo.curWindow, 20);
    Screen('TextStyle', screenInfo.curWindow, 0); % regular
    Screen('TextColor', screenInfo.curWindow, 255); % white
    
    % get experimenter and subject keyboards
    keyboardIndices = GetKeyboardIndices;
    if setup.keyB == 1 % use primary keyboard for subject and experimenter
        screenInfo.keyboard_sub = keyboardIndices(1);
        screenInfo.keyboard_exp = keyboardIndices(1);
    elseif setup.keyB == 2
        screenInfo.keyboard_sub = min(keyboardIndices);
        screenInfo.keyboard_exp = max(keyboardIndices);
    else disp('Check setup: only 1 or 2 keyboards supported.');
    end
    
    ResultMat = [];
    
    ExperimentProtocol = cell(0);
    ExperimentProtocol = [ExperimentProtocol; {'SessionOnset'}, {GetSecs}, {[]}, {[]}, {[]}, {NaN}, {[]}];

    %% Intro slide

    DrawFormattedText(screenInfo.curWindow, '\nWillkommen bei der Multi-Attribut Aufgabe\n\nIm folgenden werden Sie Punktewolken sehen, die verschiedene Charakteristika aufweisen: \n\n Bewegungsrichtung \n Helligkeit \n Gre \n Farbe \n\n Im Folgenden sehen Sie ein (lngeres) Beispiel', 'center', 'center');
    Screen('Flip', screenInfo.curWindow);
    disp('Experimenter input required: Intro Example --> Start Example');

    % wait for experimenter to resume session
    while true
        [exitSession, resumeSession] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_exp);
    if resumeSession
        break
    elseif exitSession
        closeExperiment;
        return
    end
    end
    
    %% Show first 5 second example
    
    dotInfo.durCue = 0;     % duration of cue
    dotInfo.durPres = 5;	% duration of presentation
    dotInfo.durResp = 0;	% duration of question
    dotInfo.durConf = 0;	% duration of confidence
    
    dotInfo.MAT.percAtt1H = .65;
    dotInfo.MAT.percAtt1L = 1-dotInfo.MAT.percAtt1H;
    dotInfo.MAT.percAtt2H = .65;
    dotInfo.MAT.percAtt2L = 1-dotInfo.MAT.percAtt2H;
    dotInfo.MAT.percAtt3H = .65;
    dotInfo.MAT.percAtt3L = 1-dotInfo.MAT.percAtt3H;
    dotInfo.MAT.percAtt4H = .65;
    dotInfo.MAT.percAtt4L = 1-dotInfo.MAT.percAtt4H;
    dotInfo.MAT.color = [255 255 255; 255 0 0];
    dotInfo.MAT.coherence = 1;  % define dot movement (coherence) [useless now!]
    dotInfo.MAT.direction = [180 0]; % left and right
    dotInfo.MAT.size = [5 8]; % define dot size
    dotInfo.MAT.luminance = [.4 .8]; % define dot luminance
    dotInfo.MAT.attNames = {'color'; 'direction'; 'size'; 'luminance'};
    
    TrialInitiation = GetSecs;
    ExperimentProtocol = [ExperimentProtocol; {'TrialInitiation'}, {TrialInitiation}, {[]}, {[]}, {[]}, {0}, {[]}];
    targets = makeDotTargets(screenInfo, dotInfo); % initialize fixation targets
    showTargets(screenInfo, targets, 1);
    DisplayInfo.CombPositionByTrial = [];
    DisplayInfo.CombSamples = [];
    [~, dotInfo, ExperimentProtocol,ResultMat,DisplayInfo] = dotsX_JQK_MAT_170704(screenInfo, dotInfo, targets, 1, ExperimentProtocol,ResultMat,DisplayInfo);              

    %% Instruction
    
    DrawFormattedText(screenInfo.curWindow, 'Nun sehen Sie die Attribute einzeln und in der spteren Anzeigedauer. \n Bitte geben Sie an, von welcher der beiden Optionen mehr prsentiert wurde.', 'center', 'center');
    Screen('Flip', screenInfo.curWindow);
    disp('Experimenter input required: Intro Single Attributes');
    
    pause(1);
    
    % wait for experimenter to resume session
    while true
        [exitSession, resumeSession] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_exp);
    if resumeSession
        break
    elseif exitSession
        closeExperiment;
        return
    end
    end
    
    %% Instruction
    
    DrawFormattedText(screenInfo.curWindow, 'Wir beginnen mit der Farbe. \n\n Falls die weien Punkte berwiegen, drcken Sie die linke Taste. \n Falls die roten Punkte berwiegen, dcken Sie die rechte Taste. \n\n Im Anschluss an Ihre Antwort erhalten Sie eine Rckmeldung.', 'center', 'center');
    Screen('Flip', screenInfo.curWindow);
    disp('Experimenter input required: Intro Coulour --> Start Colour presentation');

    pause(1);
    
    % wait for experimenter to resume session
    while true
        [exitSession, resumeSession] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_exp);
    if resumeSession
        break
    elseif exitSession
        closeExperiment;
        return
    end
    end
    
    %% 1st dimension
    
    % intialize psi parameters
    PM = PAL_AMPM_setupPM(...
        'priorAlphaRange' , P.PM.priorAlphaRange, ...
        'priorBetaRange', P.PM.priorBetaRange,...
        'gamma', P.PM.priorGammaRange,...
        'priorLambdaRange', P.PM.priorLambdaRange, ...
        'stimRange', P.PM.stimRange, ...
        'numtrials', P.PM.numtrials, ...
        'PF' , P.PM.PF,...
        'marginalize', 'slope', 'marginalize', 'lambda');
     
    % intiate parameters
    indTrial = 1;
    DisplayInfo.CombPositionByTrial = [];
    DisplayInfo.CombSamples = [];
    ResultMat = [];
    % run the staircase algorithm until criterion is reached
    dotInfo = [];
    dotInfo = createDotInfo_TEST_170704(PM, 1);
    dotInfo.deployed = true;
    dotInfo.feedback = 1;
    
    ExperimentProtocol = [ExperimentProtocol; {'TrialInitiation'}, {GetSecs}, {[]}, {[]}, {[]}, {0}, {[]}];
    while ~PM.stop
        % Set the difference in amount of evidence for the target feature
        % to the adaptive value. Note that for the remaining attributes,
        % this is meaningless, as the different options are equal anyways.
        disp(num2str(PM.xCurrent))
        dotInfo.MAT.percAtt1H = PM.xCurrent;
        dotInfo.MAT.percAtt1L = 1-dotInfo.MAT.percAtt1H;
        targets = makeDotTargets(screenInfo, dotInfo); % initialize fixation targets
        showTargets(screenInfo, targets, 1);
        [~, dotInfo, ExperimentProtocol,ResultMat,DisplayInfo] = dotsX_JQK_MAT_170704(screenInfo, dotInfo, targets, indTrial, ExperimentProtocol,ResultMat,DisplayInfo);              
        % update accuracy for the staircase code
        trial.resp = ResultMat(indTrial,4);
        % update PM structure
        PM = PAL_AMPM_updatePM(PM, trial.resp);
        indTrial = indTrial+1;
    end
    
    Results{1,1} = dotInfo;
    Results{2,1} = ExperimentProtocol;
    Results{3,1} = ResultMat;
    Results{4,1} = PM;
    Results{5,1} = DisplayInfo;
    
    % estimate threshold
    thresholds.color = PM.threshold(end);
    
    %% Instruction
    
    DrawFormattedText(screenInfo.curWindow, 'Als nchstes die Bewegungsrichtung. \n\n Falls die Bewegung nach links berwiegt, drcken Sie die linke Taste. \n Falls die Bewegung nach rechts berwiegt, dcken Sie die rechte Taste.', 'center', 'center', [255 255 255]);
    Screen('Flip', screenInfo.curWindow);
    disp('Experimenter input required: Intro Direction --> Start Direction presentation');

    pause(1);
    
    % wait for experimenter to resume session
    while true
        [exitSession, resumeSession] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_exp);
    if resumeSession
        break
    elseif exitSession
        closeExperiment;
        return
    end
    end
    
    %% 2nd dimension
    
    % intialize psi parameters
    PM = PAL_AMPM_setupPM(...
        'priorAlphaRange' , P.PM.priorAlphaRange, ...
        'priorBetaRange', P.PM.priorBetaRange,...
        'gamma', P.PM.priorGammaRange,...
        'priorLambdaRange', P.PM.priorLambdaRange, ...
        'stimRange', P.PM.stimRange, ...
        'numtrials', P.PM.numtrials, ...
        'PF' , P.PM.PF,...
        'marginalize', 'slope', 'marginalize', 'lambda');
     
    % intiate parameters
    indTrial = 1;
    DisplayInfo.CombPositionByTrial = [];
    DisplayInfo.CombSamples = [];
    ResultMat = [];
    % run the staircase algorithm until criterion is reached
    dotInfo = [];
    dotInfo = createDotInfo_TEST_170704(PM, 2);
    dotInfo.deployed = true;
    dotInfo.feedback = 1;
    
    ExperimentProtocol = [ExperimentProtocol; {'TrialInitiation'}, {GetSecs}, {[]}, {[]}, {[]}, {0}, {[]}];
    while ~PM.stop
        disp(num2str(PM.xCurrent))
        dotInfo.MAT.percAtt2H = PM.xCurrent;
        dotInfo.MAT.percAtt2L = 1-dotInfo.MAT.percAtt2H;
        targets = makeDotTargets(screenInfo, dotInfo); % initialize fixation targets
        showTargets(screenInfo, targets, 1);
        [~, dotInfo, ExperimentProtocol,ResultMat,DisplayInfo] = dotsX_JQK_MAT_170704(screenInfo, dotInfo, targets, indTrial, ExperimentProtocol,ResultMat,DisplayInfo);
        % update accuracy for the staircase code
        trial.resp = ResultMat(indTrial,4);
        % update PM structure
        PM = PAL_AMPM_updatePM(PM, trial.resp);
        indTrial = indTrial+1;
    end
    
    Results{1,2} = dotInfo;
    Results{2,2} = ExperimentProtocol;
    Results{3,2} = ResultMat;
    Results{4,2} = PM;
    Results{5,2} = DisplayInfo;
    
    % estimate threshold
    thresholds.direction = PM.threshold(end);
    
    %% Instruction
    
    DrawFormattedText(screenInfo.curWindow, 'Als nchstes die Punktgre. \n\n Falls die kleineren Punkte berwiegen, drcken Sie die linke Taste. \n Falls die greren Punkte berwiegen, dcken Sie die rechte Taste.', 'center', 'center', [255 255 255]);
    Screen('Flip', screenInfo.curWindow);
    disp('Experimenter input required: Intro Size --> Start Size presentation');

    pause(1);
    
    % wait for experimenter to resume session
    while true
        [exitSession, resumeSession] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_exp);
    if resumeSession
        break
    elseif exitSession
        closeExperiment;
        return
    end
    end
    
    %% 3rd dimension
    
    % intialize psi parameters
    PM = PAL_AMPM_setupPM(...
        'priorAlphaRange' , P.PM.priorAlphaRange, ...
        'priorBetaRange', P.PM.priorBetaRange,...
        'gamma', P.PM.priorGammaRange,...
        'priorLambdaRange', P.PM.priorLambdaRange, ...
        'stimRange', P.PM.stimRange, ...
        'numtrials', P.PM.numtrials, ...
        'PF' , P.PM.PF,...
        'marginalize', 'slope', 'marginalize', 'lambda');
     
    % intiate parameters
    indTrial = 1;
    DisplayInfo.CombPositionByTrial = [];
    DisplayInfo.CombSamples = [];
    ResultMat = [];
    % run the staircase algorithm until criterion is reached
    dotInfo = [];
    dotInfo = createDotInfo_TEST_170704(PM, 3);
    dotInfo.deployed = true;
    dotInfo.feedback = 1;
    
    ExperimentProtocol = [ExperimentProtocol; {'TrialInitiation'}, {GetSecs}, {[]}, {[]}, {[]}, {0}, {[]}];
    while ~PM.stop
        disp(num2str(PM.xCurrent))
        dotInfo.MAT.percAtt3H = PM.xCurrent;
        dotInfo.MAT.percAtt3L = 1-dotInfo.MAT.percAtt3H;
        targets = makeDotTargets(screenInfo, dotInfo); % initialize fixation targets
        showTargets(screenInfo, targets, 1);
        [~, dotInfo, ExperimentProtocol,ResultMat,DisplayInfo] = dotsX_JQK_MAT_170704(screenInfo, dotInfo, targets, indTrial, ExperimentProtocol,ResultMat,DisplayInfo);
        % update accuracy for the staircase code
        trial.resp = ResultMat(indTrial,4);
        % update PM structure
        PM = PAL_AMPM_updatePM(PM, trial.resp);
        indTrial = indTrial+1;
    end
    
    Results{1,3} = dotInfo;
    Results{2,3} = ExperimentProtocol;
    Results{3,3} = ResultMat;
    Results{4,3} = PM;
    Results{5,3} = DisplayInfo;
    
    % estimate threshold
    thresholds.size = PM.threshold(end);
    
    %% Instruction
    
    DrawFormattedText(screenInfo.curWindow, 'Als letztes die Punkthelligkeit. \n\n Falls die dunkleren Punkte berwiegen, drcken Sie die linke Taste. \n Falls die helleren Punkte berwiegen, dcken Sie die rechte Taste.', 'center', 'center', [255 255 255]);
    Screen('Flip', screenInfo.curWindow);
    disp('Experimenter input required: Intro Luminance --> Start Luminance presentation');

    pause(1);
    
    % wait for experimenter to resume session
    while true
        [exitSession, resumeSession] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_exp);
    if resumeSession
        break
    elseif exitSession
        closeExperiment;
        return
    end
    end
    
     %% 4th dimension
    
    % intialize psi parameters
    PM = PAL_AMPM_setupPM(...
        'priorAlphaRange' , P.PM.priorAlphaRange, ...
        'priorBetaRange', P.PM.priorBetaRange,...
        'gamma', P.PM.priorGammaRange,...
        'priorLambdaRange', P.PM.priorLambdaRange, ...
        'stimRange', P.PM.stimRange, ...
        'numtrials', P.PM.numtrials, ...
        'PF' , P.PM.PF,...
        'marginalize', 'slope', 'marginalize', 'lambda');
     
    % intiate parameters
    indTrial = 1;
    DisplayInfo.CombPositionByTrial = [];
    DisplayInfo.CombSamples = [];
    ResultMat = [];
    % run the staircase algorithm until criterion is reached
    dotInfo = [];
    dotInfo = createDotInfo_TEST_170704(PM, 4);
    dotInfo.deployed = true;
    dotInfo.feedback = 1;
    
    ExperimentProtocol = [ExperimentProtocol; {'TrialInitiation'}, {GetSecs}, {[]}, {[]}, {[]}, {0}, {[]}];
    while ~PM.stop
        disp(num2str(PM.xCurrent))
        dotInfo.MAT.percAtt4H = PM.xCurrent;
        dotInfo.MAT.percAtt4L = 1-dotInfo.MAT.percAtt4H;
        targets = makeDotTargets(screenInfo, dotInfo); % initialize fixation targets
        showTargets(screenInfo, targets, 1);
        [~, dotInfo, ExperimentProtocol,ResultMat,DisplayInfo] = dotsX_JQK_MAT_170704(screenInfo, dotInfo, targets, indTrial, ExperimentProtocol,ResultMat,DisplayInfo);        % update accuracy for the staircase code
        trial.resp = ResultMat(indTrial,4);
        % update PM structure
        PM = PAL_AMPM_updatePM(PM, trial.resp);
        indTrial = indTrial+1;
    end
    
    Results{1,4} = dotInfo;
    Results{2,4} = ExperimentProtocol;
    Results{3,4} = ResultMat;
    Results{4,4} = PM;
    Results{5,4} = DisplayInfo;
    
    % estimate threshold
    thresholds.luminance = PM.threshold(end);
    
    %% Instruction
    
    DrawFormattedText(screenInfo.curWindow, 'Nun kennen Sie alle Attribute. \n  Diese werden Ihnen im Folgenden gemeinsam prsentiert. \n Bitte geben Sie wie zuvor an, welche der beiden Optionen eines Attributs berwiegend vorhanden ist. \n Das relevante Attribut wird Ihnen dabei nach der Prsentation prsentiert.', 'center', 'center', [255 255 255]);
    Screen('Flip', screenInfo.curWindow);
    disp('Experimenter input required: Intro Realistic no cue --> Start Realistic no cue presentation');

    pause(1);
    
    % wait for experimenter to resume session
    while true
        [exitSession, resumeSession] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_exp);
    if resumeSession
        break
    elseif exitSession
        closeExperiment;
        return
    end
    end
        
    %% realistic without cue
    
    dotInfo = createDotInfo_JQK_170704(thresholds); dotInfo.deployed = true;
    dotInfo.totalTrials = 10;
    dotInfo.durCue = 0; % duration of cue
    dotInfo.feedback = 1;
    dotInfo.durConf = 0;

    ExperimentProtocol = [ExperimentProtocol; {'TrialInitiation'}, {GetSecs}, {[]}, {[]}, {[]}, {0}, {[]}];
    for indTrial = 1:dotInfo.totalTrials
        targets = makeDotTargets(screenInfo, dotInfo); % initialize fixation targets
        showTargets(screenInfo, targets, 1);
        [~, dotInfo, ExperimentProtocol,ResultMat,DisplayInfo] = dotsX_JQK_MAT_170704(screenInfo, dotInfo, targets, indTrial, ExperimentProtocol,ResultMat,DisplayInfo);
    end
    
    Results{1,5} = dotInfo;
    Results{2,5} = ExperimentProtocol;
    Results{3,5} = ResultMat;
    Results{5,5} = DisplayInfo;
    
    %% Instruction 1
    
    DrawFormattedText(screenInfo.curWindow, 'Whrend der Aufgabe werden Sie wissen, welche der Attribute abgefragt werden kann. \n Diese werden Ihnen in den Ecken des Bildschirms prsentiert. \n Die Anzahl der Attribute, die abgefragt werden knnen wird variieren. \n Sie werden nun nicht mehr erfahren, ob Ihre Antworten korrekt waren.', 'center', 'center', [255 255 255]);
    Screen('Flip', screenInfo.curWindow);
    disp('Experimenter input required: Intro Cueing, no feedback');

    pause(1);
    
    % wait for experimenter to resume session
    while true
        [exitSession, resumeSession] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_exp);
    if resumeSession
        break
    elseif exitSession
        closeExperiment;
        return
    end
    end
    
    %% Instruction 2
    
    DrawFormattedText(screenInfo.curWindow, 'Zudem bitten wir Sie im Anschluss um eine Einschtzung, wie sicher Sie sich bei der Entscheidung sind. \n  Bitte drcken Sie dann die linke Taste, falls sie sich eher unsicher sind. \n Bitte drcken Sie dann die rechte Taste, falls sie sich eher sicher sind.', 'center', 'center', [255 255 255]);
    Screen('Flip', screenInfo.curWindow);
    disp('Experimenter input required: Intro Confidence --> Start Realistic cueing, no feedback');

    pause(1);
    
    % wait for experimenter to resume session
    while true
        [exitSession, resumeSession] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_exp);
    if resumeSession
        break
    elseif exitSession
        closeExperiment;
        return
    end
    end
    
    %% realistic with cue
    
    dotInfo = createDotInfo_JQK_170809(thresholds); dotInfo.deployed = true;
    
    % select three blocks:
    
    tmp_idx = find(dotInfo.StateOrder(:,1)==4); tmp_4 = tmp_idx(1);
    tmp_idx = find(dotInfo.StateOrder(:,1)==2); tmp_2 = tmp_idx(1);
    tmp_idx = find(dotInfo.StateOrder(:,1)==1); tmp_1 = tmp_idx(1);
    
    dotInfo.StateOrder(1:3,:)       = dotInfo.StateOrder([tmp_4,tmp_2,tmp_1],:);
    dotInfo.AttCues(1:3,:)          = dotInfo.AttCues([tmp_4,tmp_2,tmp_1],:);
    dotInfo.targetAtt(1:3,:)        = dotInfo.targetAtt([tmp_4,tmp_2,tmp_1],:);
    dotInfo.targetOption(1:3,:)     = dotInfo.targetOption([tmp_4,tmp_2,tmp_1],:);
    dotInfo.HighProbChoice(1:3,:)   = dotInfo.HighProbChoice([tmp_4,tmp_2,tmp_1],:);
    
    dotInfo.blockAmount = 3;
    
    for indBlock = 1:dotInfo.blockAmount
        
          % present state cues
            for indAtt = 1:4
                CueImg = ['img/', dotInfo.MAT.attNames{indAtt},'?.png'];
                [cueLoad,~,~] = imread(CueImg);
                cue2Disp = Screen('MakeTexture',screenInfo.curWindow,cueLoad);
                smallIm = [0 0 floor(size(cueLoad,2)/1.5) floor(size(cueLoad,1)/1.5)];
                switch indAtt 
                    case 1
                        smallIm = CenterRect([0 0 floor(size(cueLoad,2)/1.5) floor(size(cueLoad,1)/1.5)], [0 0 screenInfo.screenRect(3)/2 screenInfo.screenRect(4)/2]);
                    case 2
                        smallIm = CenterRect([0 0 floor(size(cueLoad,2)/1.5) floor(size(cueLoad,1)/1.5)], [screenInfo.screenRect(3)/2 0 screenInfo.screenRect(3) screenInfo.screenRect(4)/2]);
                    case 3
                        smallIm = CenterRect([0 0 floor(size(cueLoad,2)/1.5) floor(size(cueLoad,1)/1.5)], [0 screenInfo.screenRect(4)/2 screenInfo.screenRect(3)/2 screenInfo.screenRect(4)]);
                    case 4
                        smallIm = CenterRect([0 0 floor(size(cueLoad,2)/1.5) floor(size(cueLoad,1)/1.5)], [screenInfo.screenRect(3)/2 screenInfo.screenRect(4)/2 screenInfo.screenRect(3) screenInfo.screenRect(4)]);
                end
                if ismember(indAtt, dotInfo.AttCues{indBlock,1})
                    Screen('DrawTexture', screenInfo.curWindow, cue2Disp, [], smallIm); % draw the object
                end;
            end
            oldTextSize = Screen('TextSize', screenInfo.curWindow, 30);
            DrawFormattedText(screenInfo.curWindow, ['relevante Attribute: ', num2str(dotInfo.StateOrder(indBlock,1))], 'center', 'center');
            Screen('TextSize', screenInfo.curWindow, oldTextSize); clear oldTextSize;
            BlockInitiation = Screen('Flip', screenInfo.curWindow);
            ExperimentProtocol = cell(0);
            ExperimentProtocol = [ExperimentProtocol; {'BlockInitiation'}, {BlockInitiation}, {[]}, {[]}, {[]}, {indBlock}, {[]}, {[]}];
            pause(dotInfo.durBlockOnset);
          
          for indTrial = 1:dotInfo.blockLengthDim
                fprintf('Block %i of %i; Trial %i of %i\n', [indBlock, dotInfo.blockAmount, indTrial, dotInfo.blockLengthDim])
                TrialInitiation = GetSecs;
                ExperimentProtocol = [ExperimentProtocol; {'TrialInitiation'}, {TrialInitiation}, {[]}, {[]}, {[]}, {indBlock}, {indTrial}, {[]}];
                % create fixation
                targets = makeDotTargets(screenInfo, dotInfo); % initialize targets
                %showTargets(screenInfo, targets, 1);
                [~, dotInfo, ExperimentProtocol,ResultMat,DisplayInfo] = dotsX_JQK_MAT_170809(screenInfo, dotInfo, targets, indBlock, indTrial, ExperimentProtocol,ResultMat,DisplayInfo);
          end % trial presentation
      end % block presentation
    
    Results{1,6} = dotInfo;
    Results{2,6} = ExperimentProtocol;
    Results{3,6} = ResultMat;
    Results{5,6} = DisplayInfo;
    
    save(sessionFile, 'Results', 'thresholds');
    
    %% inform the subject that the experiment is over
    
    DrawFormattedText(screenInfo.curWindow, 'Sie haben das Training erfolgreich beendet!', 'center', 'center', [255 255 255]);
    Screen('Flip', screenInfo.curWindow);

    pause(1);
    
    %% plot threshold data for threshold optimization of MAT task

    h = figure;
    subplot(3,1,1);
    plot(Results{4,1}.threshold)
    hold on; plot(Results{4,2}.threshold)
    hold on; plot(Results{4,3}.threshold)
    hold on; plot(Results{4,4}.threshold)
    ylim([.5 .8]), xlim([1 Results{4,1}.numTrials])
    title('Threshold estimates');
    legend({'Color'; 'Direction'; 'Size'; 'Luminance'});

    subplot(3,1,2);
    plot(Results{4,1}.x)
    hold on; plot(Results{4,2}.x)
    hold on; plot(Results{4,3}.x)
    hold on; plot(Results{4,4}.x)
    ylim([.5 .8]), xlim([1 Results{4,1}.numTrials])
    title('Presented values');
    legend({'Color'; 'Direction'; 'Size'; 'Luminance'});

    subplot(3,1,3);
    plot(Results{4,1}.response)
    hold on; plot(Results{4,2}.response)
    hold on; plot(Results{4,3}.response)
    hold on; plot(Results{4,4}.response)
    ylim([0 1]), xlim([1 Results{4,1}.numTrials])
    title('Accuracy');
    legend({'Color'; 'Direction'; 'Size'; 'Luminance'});
    
    % Maybe this should be saved as output? Then again, we can always
    % reproduce it.
    
    %% wait for experimenter to close screen

    disp('Experimenter input required: close experiment');
    
    while true
      [exitKeyPressed, resumeKeyPressed] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_exp);
      if exitKeyPressed || resumeKeyPressed
        break
      end
    end

  catch exception
    getReport(exception) % show stack trace
  end

  closeExperiment; % close drawing canvas
  Screen('Preference', 'Verbosity', oldVerbosityLevel); % restore verbosity

end
