%_______________________________________________________________________
%
% Run random dot motion experiment defined in createDotInfo
%_______________________________________________________________________
%
% Input
%
% dotInfo | experiment configuration (struct)
%    subj | identifier of subject (string)
% condSet | conditions (cell array of strings)
%_______________________________________________________________________
%

% adapted from dotsExperiment_JQK_170327
% 170704    | cleanup, adapted
% 170707    | removed saving for each trial; instead save in catch part
%           | now saves DisplayInfo
% 170809    | adapted to state space version
% 170814    | added triggers (MR, EEG, ET); turned into run version

function MAT_experiment_170814(dotInfo, subj, setup)

    %%
    %% sanitize function parameters
    %%
    if nargin == 0
        eval('help dotsExperiment')
        return
    end

    if not(exist('dotInfo','var'))
        error('Missing input: dotInfo')
    elseif isempty(dotInfo) || not(isstruct(dotInfo))
        error('Invalid input: dotInfo (requires non-empty struct)')
    end

    if not(exist('subj','var'))
        error('Missing input: subj')
    elseif not(isempty(subj)) && ischar(subj)
        dotInfo.subj = subj;
    else
        error('Invalid input: subj (requires non-empty string)')
    end

    if not(isfield(dotInfo,'deployed'))
        dotInfo.deployed = true;
    end
        
    %% for EEG: set up parallel port triggers

    if setup.EEG
        if setup.DIO.parallelTrigger == 1
            daqreset;
            rehash toolboxcache
           % create parallel port triggers
            for indTrigger = 1:11
                dio{indTrigger} = digitalio('parallel');
                addline(dio{indTrigger},indTrigger-1,0,'out');
                setup.DIO.uddobj{indTrigger} = daqgetfield(dio{indTrigger},'uddobject')
            end
            setup.DIO.uddobj1 = setup.DIO.uddobj{1}; % Start/End trigger (1-4)
            setup.DIO.uddobj2 = setup.DIO.uddobj{2}; % Start/End trigger (1-4)
            setup.DIO.uddobj3 = setup.DIO.uddobj{3}; % Start/End trigger (1-4)
            setup.DIO.uddobj4 = setup.DIO.uddobj{4}; % Start/End trigger (1-4)
            setup.DIO.uddobj5 = setup.DIO.uddobj{5}; % block cue onset trigger
            setup.DIO.uddobj6 = setup.DIO.uddobj{6}; % cue onset trigger
            setup.DIO.uddobj7 = setup.DIO.uddobj{7}; % stim onset trigger
            setup.DIO.uddobj8 = setup.DIO.uddobj{8}; % acc query onset trigger
            setup.DIO.uddobj9 = setup.DIO.uddobj{9}; % response acc trigger
            setup.DIO.uddobj10 = setup.DIO.uddobj{10}; % acc query onset trigger
            setup.DIO.uddobj11 = setup.DIO.uddobj{11}; % response conf trigger
        end
    end
    
  %% prepare MR
  
    if setup.MR
        try
            outportb(890,32) % sets  pin for tristate in base+2 to up state. This allows to read from the port
        catch
            disp('outportb(890,32) unsuccessful');
        end
    end
    
    %% set behavioural data savepath

    sessionFile = [dotInfo.dataDir, subj '_' datestr(now, 'yymmdd_HHMM'), '.mat'];

    %% set up PTB

    displayList = Screen('screens');
    displayIdx = displayList(2);
    KbName('UnifyKeyNames'); % Make sure keyboard mapping is the same on all supported operating systems
    oldVerbosityLevel = Screen('Preference', 'Verbosity', 2); % show errors and warnings
  
    if setup.DEBUG == 1
        Screen('Preference', 'SkipSyncTests', 1);
        PsychDebugWindowConfiguration(0, setup.opacity);
    else
        clear Screen; %PsychDebugWindowConfiguration([], 1);
    end
    
    % Do dummy calls to GetSecs, WaitSecs, KbCheck(-1) to make sure
    % they are loaded and ready when we need them - without delays
    % in the wrong moment:
    KbCheck(-1);
    WaitSecs(0.1);
    GetSecs;

  try

    %% prepare canvas

    screenInfo = openExperiment(50,50,displayIdx); clc; % open drawing canvas
    if numel(Screen('screens')) == 1 || dotInfo.deployed
      HideCursor(screenInfo.curWindow);
    end
    Screen('TextFont', screenInfo.curWindow, 'Helvetica');
    Screen('TextSize', screenInfo.curWindow, 20);
    Screen('TextStyle', screenInfo.curWindow, 0); % regular
    Screen('TextColor', screenInfo.curWindow, 255); % white
    ifi = Screen('GetFlipInterval', screenInfo.curWindow); % Query the frame duration

    % set up experimenter and subject keyboards
    keyboardIndices = GetKeyboardIndices;
    if setup.keyB == 1 % use primary keyboard for subject and experimenter
        screenInfo.keyboard_sub = keyboardIndices(1);
        screenInfo.keyboard_exp = keyboardIndices(1);
    elseif setup.keyB == 2
        screenInfo.keyboard_sub = min(keyboardIndices);
        screenInfo.keyboard_exp = max(keyboardIndices);
    else disp('Check setup: only 1 or 2 keyboards supported.');
    end
        
    %% startup ET
    
    if setup.ET
        % Initialize eyelink defaults and control code structure
        el=EyelinkInitDefaults(screenInfo.curWindow);
        el.wRect = wRect;
        [winWidth, winHeight] = WindowSize(screenInfo.curWindow);
        el.winWidth  = winWidth;
        el.winHeight = winHeight;
    else el = [];
    end
    
    %% session start info
    
    Timing.sessStartTime = GetSecs;

    ResultMat = NaN(dotInfo.totalTrials,4);
    DisplayInfo = [];
    ExperimentProtocol = cell(0);
    ExperimentProtocol = [ExperimentProtocol; {'SessionOnset'}, {Timing.sessStartTime}, {[]}, {[]},{[]} {[]},{NaN}, {[]}];
    
    %% run loop
            
    for indRun = 1:dotInfo.runAmount
        
        if setup.ET
            Screen('TextSize', screenInfo.curWindow, 20);
            DrawFormattedText(screenInfo.curWindow, ...
                'Wir werden nun den Eyetracker konfigurieren. \n\n Bitte fixieren Sie zunchst die weien, hohlen Kreise, die auf dem Bildschirm auftauchen.', ...
                'center', 'center', [255 255 255 255]);
            Screen('Flip',screenInfo.curWindow);
            subNo = subj(1:4); % extract first four characters for subject ID
            phase = 1;
            el = ET_setup_JQK(setup.ELdummymode, el, subNo, phase, indRun);  % initialize the eye tracker and start recording
            % The protocol should have the following format:
            % [subject, time, run, block, trial, event]
            Eyelink('Message', sprintf('Subj%s Time%d %Run%d Block%d Trial%d Start', ...
                subj, ...
                GetSecs(),...
                indRun, ...
                0, ...
                0));
        end
        
        if setup.EEG
            %% send START parallelTrigger
            if setup.DIO.parallelTrigger == 1
                % first trigger (1)
                putvalue(setup.DIO.uddobj{1},0,1); % set to 0 (trigger off); ~20us (undocumented use demo in @dioline\putvalue.m and @digitalio\putvalue.m - args are: uddobj, vals [, lineInds])
                WaitSecs(0.5);
                setup.DIO.StartTrigger_101 = getvalue(setup.DIO.uddobj{1},1);
                putvalue(setup.DIO.uddobj{1},1,1); % set to 1 (trigger on)
                WaitSecs(0.5);
                setup.DIO.StartTrigger_111 = getvalue(setup.DIO.uddobj{1},1);
                putvalue(setup.DIO.uddobj{1},0,1); % set to 0 (trigger off)
                WaitSecs(0.5);
                setup.DIO.StartTrigger_102 = getvalue(setup.DIO.uddobj{1},1);
                % second trigger (2)
                putvalue(setup.DIO.uddobj{2},0,1); % set to 0 (trigger off)
                WaitSecs(0.5);
                setup.DIO.StartTrigger_201 = getvalue(setup.DIO.uddobj{2},1);
                putvalue(setup.DIO.uddobj{2},1,1); % set to 1 (trigger on)
                WaitSecs(0.5);
                setup.DIO.StartTrigger_211 = getvalue(setup.DIO.uddobj{2},1);
                putvalue(setup.DIO.uddobj{2},0,1); % set to 0 (trigger off)
                WaitSecs(0.5);
                setup.DIO.StartTrigger_202 = getvalue(setup.DIO.uddobj{2},1);
            end
        end

        % put Ready... on the screen and wait for user presses Return
        DrawFormattedText(screenInfo.curWindow, 'Bereit?', 'center', 'center');
        Screen('Flip', screenInfo.curWindow);
        while true
            [exitSession, resumeSession] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_sub);
        if resumeSession
            if setup.ET
                Eyelink('Message', sprintf('Run %d Ready screen onset', indRun));
            end
            disp('Subject input required: ready for presentation? --> State switch start');
            break
        elseif exitSession
            shutdownStudy(el);
            return
        end
        end
        
        if setup.MR % wait for scanner pulse to start exp
            DrawFormattedText(screenInfo.curWindow, 'Waiting for scanner pulse','center','center', WhiteIndex(w))
            Screen('Flip', screenInfo.curWindow);
            try
                scannerPort     = 888;
                triggerSwitches = 17; % TR of experiment onset; wait for 17th TR (at TR of .625s, this should allow for 10s of equilibration)
                [SCAN t0] = fMRI_waitScannerTriggers(scannerPort, irun, triggerSwitches, []);
                t0 = GetSecs;
                Eyelink('Message', sprintf('Run%d fMRI scanning Start time %g', irun, t0));
            catch
                warning('MR Triggers not working!!')
            end
        else
            t0 = GetSecs;
        end
        
        %%%%%%%%%%% START RUN %%%%%%%%%%%%%%%%%
        Timing.RunInitiation = t0; Timing.lastTiming = 'NaN';
        tic; % from now on scanning
        ExperimentProtocol = [ExperimentProtocol; {'RunInitiation'}, {Timing.RunInitiation}, {[]}, {[]}, {[]}, {indRun}, {[]}, {[]}];
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
        %% block loop
        
        for indBlock = 1:dotInfo.blocksPerRun
            % initialize state cues of current condition
            addSurroundingCues(dotInfo, screenInfo, indRun, indBlock, 1);
            oldTextSize = Screen('TextSize', screenInfo.curWindow, 30);
            DrawFormattedText(screenInfo.curWindow, ['relevante Attribute: ', num2str(dotInfo.StateOrderRun{indRun}(indBlock,1))], 'center', 'center');
            Screen('TextSize', screenInfo.curWindow, oldTextSize); clear oldTextSize;
            
            % ####### Block cue onset #####
            if strcmp(dotInfo.timing, 'absolute')
                flipWhen = Timing.RunInitiation+(indBlock-1)*dotInfo.durBlockOnset+(indBlock-1)*dotInfo.blockLengthDim*dotInfo.trialDuration.all-(ifi/2);
            else
                if strcmp(Timing.lastTiming, 'NaN')
                    flipWhen = 0;
                elseif strcmp(Timing.lastTiming, 'ITI')
                    flipWhen = Timing.ITIOnset+dotInfo.durITI-(ifi/2);
                end
            end
            Timing.BlockInitiation = Screen('Flip', screenInfo.curWindow, flipWhen);
            Timing.lastTiming = 'BlockCue';
            ExperimentProtocol = [ExperimentProtocol; {'BlockInitiation'}, {Timing.BlockInitiation}, {[]}, {[]}, {[]}, {indBlock}, {[]}, {[]}];
            if setup.ET
                Eyelink('Message', sprintf('Block %d BlockCueOnset', indBlock));
            end
            if setup.EEG
                if setup.DIO.parallelTrigger == 1
                    % set to off state
                    putvalue(setup.DIO.uddobj{5},0,1);
                    WaitSecs(0.1)
                    setup.DIO.BlockCueTrigger_01(indBlock) = getvalue(setup.DIO.uddobj{5},1);
                    % set to on state
                    putvalue(setup.DIO.uddobj{5},1,1);
                    WaitSecs(0.1)
                    setup.DIO.BlockCueTrigger_11(indBlock) = getvalue(setup.DIO.uddobj{5},1);
                    % set to off state
                    putvalue(setup.DIO.uddobj{5},0,1);
                    WaitSecs(0.1)
                    setup.DIO.BlockCueTrigger_02(indBlock) = getvalue(setup.DIO.uddobj{5},1);
                end
            end
            % #############################

            for indTrial = 1:dotInfo.blockLengthDim
                if setup.ET
                    % Sending a 'TRIALID' message to mark the start of a trial in Data
                    % Viewer.  This is different than the start of recording message
                    % START that is logged when the trial recording begins. The viewer
                    % will not parse any messages, events, or samples, that exist in
                    % the data file prior to this message.
                    Eyelink('Message', 'BLOCKID %d TRIALID %d', indBlock, indTrial);
                    Eyelink('command', 'record_status_message "RUN%d BLOCK%d / %d TRIAL %d / %d"', ...
                        indRun, indBlock, dotInfo.blocksPerRun, indTrial, dotInfo.blockLengthDim);  % This supplies the title at the bottom of the eyetracker display  
                end
                fprintf('Run %i of %i; Block %i of %i; Trial %i of %i\n', ...
                    [indRun, dotInfo.runAmount, indBlock, dotInfo.blocksPerRun, indTrial, dotInfo.blockLengthDim])
                Timing.TrialInitiation = GetSecs;
                ExperimentProtocol = [ExperimentProtocol; {'TrialInitiation'}, {Timing.TrialInitiation}, {[]}, {[]}, {[]}, {indBlock}, {indTrial}, {[]}];
                % create fixation
                targets = makeDotTargets(screenInfo, dotInfo); % initialize targets
                %showTargets(screenInfo, targets, 1);
                [~, dotInfo, ExperimentProtocol,ResultMat,DisplayInfo,Timing] = ...
                    dotsX_JQK_MAT_170814(screenInfo, dotInfo, targets, indRun, indBlock, indTrial, ...
                    ExperimentProtocol,ResultMat,DisplayInfo,Timing,setup);
                % ############ 'please blink' ITI #########################
                if strcmp(dotInfo.timing, 'absolute')
                    flipWhen = Timing.RunInitiation+(indBlock)*dotInfo.durBlockOnset+(indBlock-1)*dotInfo.blockLengthDim*dotInfo.trialDuration.all+(indTrial-1)*dotInfo.trialDuration.all+dotInfo.trialDuration.Reward-(ifi/2);
                else
                    flipWhen = Timing.ConfOnset+dotInfo.durConf-(ifi/2);
                end
                % add cues
                addSurroundingCues(dotInfo, screenInfo, indRun, indBlock, indTrial);
                DrawFormattedText(screenInfo.curWindow, 'Bitte blinzeln.', 'center', 'center');
                Timing.ITIOnset = Screen('Flip', screenInfo.curWindow, flipWhen);
                ExperimentProtocol = [ExperimentProtocol; {'ITIOnset'}, {Timing.ITIOnset}, {[]}, {[]}, {[]}, {indBlock}, {indTrial}, {[]}];
                Timing.lastTiming = 'ITI';
                if setup.ET
                    Eyelink('Message', sprintf('Block %d Trial %d ITIOnset', indBlock));
                end
                % #########################################################
            end % trial presentation
        end % block presentation
        
        %% do a break following the end of the run
        
        Timing.PauseStart = GetSecs(); isterminated = 0;
        ExperimentProtocol = [ExperimentProtocol; {'PauseStart'}, {Timing.PauseStart}, {[]}, {[]}, {[]}, {indBlock}, {indTrial}, {[]}];
        while (GetSecs()-PauseStart) < dotInfo.breakTime
            if setup.ET
                %% ############### stop ET & EEG after 10 s ####################
                if (GetSecs()-PauseStart) >= 10 && isterminated == 0
                    ExperimentProtocol = [ExperimentProtocol; {'RunEnd'}, {GetSecs()}, {[]}, {[]}, {[]}, {indBlock}, {indTrial}, {[]}];
                    toc % end scanning
                    if setup.ET && isfield(el,'vsn')
                        ET_terminate(el, basepath);  % Save et data after each run
                    end
                    if setup.EEG
                        %% send END parallelTrigger
                        if setup.DIO.parallelTrigger == 1
                            % first trigger
                            putvalue(setup.DIO.uddobj{3},0,1); % off
                            WaitSecs(0.5);
                            setup.DIO.EndTrigger_101 = getvalue(setup.DIO.uddobj{3},1);
                            putvalue(setup.DIO.uddobj{3},1,1); % on
                            WaitSecs(0.5);
                            setup.DIO.EndTrigger_111 = getvalue(setup.DIO.uddobj{3},1);
                            putvalue(setup.DIO.uddobj{3},0,1); % off
                            WaitSecs(0.5);
                            setup.DIO.EndTrigger_102 = getvalue(setup.DIO.uddobj{3},1);
                            % second trigger
                            putvalue(setup.DIO.uddobj{4},0,1); % off
                            WaitSecs(0.5);
                            setup.DIO.EndTrigger_201 = getvalue(setup.DIO.uddobj{4},1);
                            putvalue(setup.DIO.uddobj{4},1,1); % on
                            WaitSecs(0.5);
                            setup.DIO.EndTrigger_211 = getvalue(setup.DIO.uddobj{4},1);
                            putvalue(setup.DIO.uddobj{4},0,1); % off
                            WaitSecs(0.5);
                            setup.DIO.EndTrigger_202 = getvalue(setup.DIO.uddobj{4},1);
                        end
                    end
                    isterminated = 1;
                end
                % ####################################################################
            end
            % have the subject get ready for the next condition
            pause(.1);
            remain = dotInfo.breakTime-round(GetSecs - PauseStart,0);
            DrawFormattedText(screenInfo.curWindow, ['Pause: Es geht in ',num2str(remain),' Sekunden mit dem nchsten Block weiter. /n/n Sie knnen sich nun etwas ausruhen.'], 'center', 'center');
            Screen('Flip', screenInfo.curWindow);
        end
        
    end % run presentation
          
    Timing.sessEndTime = GetSecs;
    ExperimentProtocol = [ExperimentProtocol; {'SessionEnd'}, {Timing.sessEndTime}, {[]}, {[]}, {[]}, {[]}, {[]}, {[]}];

    save(sessionFile, 'ExperimentProtocol', 'dotInfo', 'ResultMat', 'DisplayInfo');

    % inform the subject that the experiment is over
    DrawFormattedText(screenInfo.curWindow, 'task complete!', 'center', 'center');
    Screen('Flip', screenInfo.curWindow);
    disp('Experimenter input required: MAT complete --> presentation offset');

%     sessTime = GetSecs - sessStartTime;
%     meanFixTime = mean([trialData.fixTime]);
%     numCorrect = numel(find([trialData.correct]));
%     numTrial = numel([trialData.correct]);
%     numResponded = numel(find(~isnan([trialData.rxtime])));
%   
%     fprintf('\nTotal Time: %.2f\n', sessTime) % incl. block startup waits
%     fprintf('Average fixation: %.2fs\n', meanFixTime)
%     fprintf('Responded: %i/%i\n', numResponded, numTrial)
%     fprintf('Correct: %i/%i\n', numCorrect, numTrial)
%     fprintf('Accuracy: %.1f\n', 100*numCorrect/numResponded)

    % wait for experimenter to close screen
    while true
        [exitKeyPressed, resumeKeyPressed] = checkKeys_byKeyB(dotInfo, screenInfo.keyboard_exp);
        if exitKeyPressed || resumeKeyPressed
            break
        end
    end
    shutdownStudy(el); % close drawing canvas
    Screen('Preference', 'Verbosity', oldVerbosityLevel); % restore verbosity
  
  catch exception
    getReport(exception) % show stack trace
    save(sessionFile);
    shutdownStudy(el); % close drawing canvas
    Priority(0);
  end % TRY...CATCH
  
end % function

function shutdownStudy(el)
    if isfield(el,'vsn')
        ET_terminate(el);
    end
    Screen('CloseAll');
    ShowCursor;
    Screen('Preference', 'SkipSyncTests', 0);
    Screen('Preference', 'Verbosity', 3);
    fprintf('\n');
end
